@@ -1,5 +1,5 @@
 #!/bin/sh
-# $Id: package.sh 8596 2006-10-05 03:25:48Z phobos $
+# $Id: package.sh 10012 2007-04-24 01:24:38Z phobos $
 # Copyright 2004-2005 Nick Mathewson. 
 # See LICENSE in Tor distribution for licensing information.
 
@@ -18,7 +18,7 @@
 
 # Where have we put the zip file containing Privoxy?  Edit this if your
 # privoxy lives somewhere else.
-PRIVOXY_PKG_ZIP=~/tmp/privoxyosx_setup_3.0.3.zip
+PRIVOXY_PKG_ZIP=~/tmp/privoxyosx_setup_3.0.6.zip
 
 ###
 # Helpful info on OS X packaging:
@@ -79,7 +79,6 @@ chmod 755 contrib/osx/TorPostflight
 cp contrib/osx/TorPostflight $BUILD_DIR/tor_resources/postflight
 cp contrib/osx/addsysuser $BUILD_DIR/tor_resources/addsysuser
 cp contrib/osx/Tor_Uninstaller.applescript $BUILD_DIR/tor_resources/Tor_Uninstaller.applescript
-cp contrib/osx/Tor_Uninstaller.app.tar.gz $BUILD_DIR/tor_resources/Tor_Uninstaller.app.tar.gz
 cp contrib/osx/uninstall_tor_bundle.sh $BUILD_DIR/tor_resources/uninstall_tor_bundle.sh
 cp contrib/osx/package_list.txt $BUILD_DIR/tor_resources/package_list.txt
 cp contrib/osx/tor_logo.gif $BUILD_DIR/tor_resources/background.gif
@@ -105,7 +104,15 @@ cp AUTHORS $DOC/AUTHORS.txt
 groff doc/tor.1.in -T ps -m man | pstopdf -i -o $DOC/tor-reference.pdf
 groff doc/tor-resolve.1 -T ps -m man | pstopdf -i -o $DOC/tor-resolve.pdf
 mkdir $DOC/Advanced
-cp doc/tor-spec.txt doc/rend-spec.txt doc/control-spec.txt doc/socks-extensions.txt doc/version-spec.txt $DOC/Advanced
+cp	doc/spec/tor-spec.txt         \
+	doc/spec/rend-spec.txt        \
+	doc/spec/control-spec.txt     \
+	doc/spec/socks-extensions.txt \
+	doc/spec/version-spec.txt     \
+	doc/spec/address-spec.txt     \
+	doc/spec/path-spec.txt        \
+	$DOC/Advanced
+
 cp doc/HACKING $DOC/Advanced/HACKING.txt
 cp ChangeLog $DOC/Advanced/ChangeLog.txt
 
