@@ -1,9 +1,9 @@
 /* Copyright 2003-2004 Roger Dingledine
  * Copyright 2004-2007 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: compat.c 9688 2007-02-28 20:24:27Z nickm $ */
+/* $Id: compat.c 10807 2007-07-12 16:17:23Z nickm $ */
 const char compat_c_id[] =
-  "$Id: compat.c 9688 2007-02-28 20:24:27Z nickm $";
+  "$Id: compat.c 10807 2007-07-12 16:17:23Z nickm $";
 
 /**
  * \file compat.c
@@ -199,8 +199,8 @@ tor_mmap_file(const char *filename)
   res->mmap_handle = NULL;
 
   res->file_handle = CreateFile(filename,
-                                GENERIC_READ,
-                                0, NULL,
+                                GENERIC_READ, FILE_SHARE_READ,
+                                NULL,
                                 OPEN_EXISTING,
                                 FILE_ATTRIBUTE_NORMAL,
                                 0);
