@@ -1,10 +1,10 @@
 /* Copyright (c) 2001 Matej Pfajfar.
  * Copyright (c) 2001-2004, Roger Dingledine.
- * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson. */
+ * Copyright (c) 2004-2007, Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: crypto.c 8652 2006-10-08 20:01:18Z nickm $ */
+/* $Id: crypto.c 9688 2007-02-28 20:24:27Z nickm $ */
 const char crypto_c_id[] =
-  "$Id: crypto.c 8652 2006-10-08 20:01:18Z nickm $";
+  "$Id: crypto.c 9688 2007-02-28 20:24:27Z nickm $";
 
 /**
  * \file crypto.c
@@ -77,8 +77,10 @@ const char crypto_c_id[] =
 #define PRIVATE_KEY_OK(k) ((k) && (k)->key && (k)->key->p)
 
 #ifdef TOR_IS_MULTITHREADED
+/** A number of prealloced mutexes for use by openssl. */
 static tor_mutex_t **_openssl_mutexes = NULL;
-static int _n_openssl_mutexes = -1;
+/** How many mutexes have we allocated for use by openssl? */
+static int _n_openssl_mutexes = 0;
 #endif
 
 /** A public key, or a public/private keypair. */
@@ -103,7 +105,6 @@ struct crypto_dh_env_t {
 
 /* Prototypes for functions only used by tortls.c */
 crypto_pk_env_t *_crypto_new_pk_env_rsa(RSA *rsa);
-RSA *_crypto_pk_env_get_rsa(crypto_pk_env_t *env);
 EVP_PKEY *_crypto_pk_env_get_evp_pkey(crypto_pk_env_t *env, int private);
 DH *_crypto_dh_env_get_dh(crypto_dh_env_t *dh);
 
@@ -154,6 +155,8 @@ crypto_log_errors(int severity, const char *doing)
     lib = (const char*)ERR_lib_error_string(err);
     func = (const char*)ERR_func_error_string(err);
     if (!msg) msg = "(null)";
+    if (!lib) lib = "(null)";
+    if (!func) func = "(null)";
     if (doing) {
       log(severity, LD_CRYPTO, "crypto error while %s: %s (in %s:%s)",
           doing, msg, lib, func);
@@ -164,6 +167,7 @@ crypto_log_errors(int severity, const char *doing)
 }
 
 #ifndef NO_ENGINES
+/** Log any OpenSSL engines we're using at NOTICE. */
 static void
 log_engine(const char *fn, ENGINE *e)
 {
@@ -189,6 +193,10 @@ crypto_global_init(int useAccel)
     OpenSSL_add_all_algorithms();
     _crypto_global_initialized = 1;
     setup_openssl_threading();
+    /* XXX the below is a bug, since we can't know if we're supposed
+     * to be using hardware acceleration or not. we should arrange
+     * for this function to be called before init_keys. But make it
+     * not complain loudly, at least until we make acceleration work. */
     if (useAccel < 0) {
       log_info(LD_CRYPTO, "Initializing OpenSSL via tor_tls_init().");
     }
@@ -260,13 +268,6 @@ _crypto_new_pk_env_rsa(RSA *rsa)
   return env;
 }
 
-/** used by tortls.c: return the RSA* from a crypto_pk_env_t. */
-RSA *
-_crypto_pk_env_get_rsa(crypto_pk_env_t *env)
-{
-  return env->key;
-}
-
 /** used by tortls.c: get an equivalent EVP_PKEY* for a crypto_pk_env_t.  Iff
  * private is set, include the private-key portion of the key. */
 EVP_PKEY *
@@ -453,7 +454,7 @@ crypto_pk_read_private_key_from_filename(crypto_pk_env_t *env,
   int r;
 
   /* Read the file into a string. */
-  contents = read_file_to_str(keyfile, 0);
+  contents = read_file_to_str(keyfile, 0, NULL);
   if (!contents) {
     log_warn(LD_CRYPTO, "Error reading private key from \"%s\"", keyfile);
     return -1;
@@ -505,7 +506,7 @@ crypto_pk_write_public_key_to_string(crypto_pk_env_t *env, char **dest,
   tor_assert(buf->length >= 0);
   *dest = tor_malloc(buf->length+1);
   memcpy(*dest, buf->data, buf->length);
-  (*dest)[buf->length] = 0; /* null terminate it */
+  (*dest)[buf->length] = 0; /* nul terminate it */
   *len = buf->length;
   BUF_MEM_free(buf);
 
@@ -541,7 +542,7 @@ crypto_pk_read_public_key_from_string(crypto_pk_env_t *env, const char *src,
   return 0;
 }
 
-/* Write the private key from 'env' into the file named by 'fname',
+/** Write the private key from <b>env</b> into the file named by <b>fname</b>,
  * PEM-encoded.  Return 0 on success, -1 on failure.
  */
 int
@@ -575,70 +576,6 @@ crypto_pk_write_private_key_to_filename(crypto_pk_env_t *env,
   return r;
 }
 
-/** Allocate a new string in *<b>out</b>, containing the public portion of the
- * RSA key in <b>env</b>, encoded first with DER, then in base-64.  Return the
- * length of the encoded representation on success, and -1 on failure.
- *
- * <i>This function is for temporary use only.  We need a simple
- * one-line representation for keys to work around a bug in parsing
- * directories containing "opt keyword\n-----BEGIN OBJECT----" entries
- * in versions of Tor up to 0.0.9pre2.</i>
- */
-int
-crypto_pk_DER64_encode_public_key(crypto_pk_env_t *env, char **out)
-{
-  int len;
-  char buf[PK_BYTES*2]; /* Too long, but hey, stacks are big. */
-  tor_assert(env);
-  tor_assert(out);
-  len = crypto_pk_asn1_encode(env, buf, sizeof(buf));
-  if (len < 0) {
-    return -1;
-  }
-  *out = tor_malloc(len * 2); /* too long, but safe. */
-  if (base64_encode(*out, len*2, buf, len) < 0) {
-    log_warn(LD_CRYPTO, "Error base64-encoding DER-encoded key");
-    tor_free(*out);
-    return -1;
-  }
-  /* Remove spaces */
-  tor_strstrip(*out, " \r\n\t");
-  return strlen(*out);
-}
-
-/** Decode a base-64 encoded DER representation of an RSA key from <b>in</b>,
- * and store the result in <b>env</b>.  Return 0 on success, -1 on failure.
- *
- * <i>This function is for temporary use only.  We need a simple
- * one-line representation for keys to work around a bug in parsing
- * directories containing "opt keyword\n-----BEGIN OBJECT----" entries
- * in versions of Tor up to 0.0.9pre2.</i>
- */
-crypto_pk_env_t *
-crypto_pk_DER64_decode_public_key(const char *in)
-{
-  char partitioned[PK_BYTES*2 + 16];
-  char buf[PK_BYTES*2];
-  int len;
-  tor_assert(in);
-  len = strlen(in);
-
-  if (strlen(in) > PK_BYTES*2) {
-    return NULL;
-  }
-  /* base64_decode doesn't work unless we insert linebreaks every 64
-   * characters.  how dumb. */
-  if (tor_strpartition(partitioned, sizeof(partitioned), in, "\n", 64,
-                       ALWAYS_TERMINATE))
-    return NULL;
-  len = base64_decode(buf, sizeof(buf), partitioned, strlen(partitioned));
-  if (len<0) {
-    log_warn(LD_CRYPTO,"Error base-64 decoding key");
-    return NULL;
-  }
-  return crypto_pk_asn1_decode(buf, len);
-}
-
 /** Return true iff <b>env</b> has a valid key.
  */
 int
@@ -791,7 +728,7 @@ crypto_pk_public_checksig_digest(crypto_pk_env_t *env, const char *data,
   tor_assert(sig);
 
   if (crypto_digest(digest,data,datalen)<0) {
-    log_warn(LD_CRYPTO, "couldn't compute digest");
+    log_warn(LD_BUG, "couldn't compute digest");
     return -1;
   }
   r = crypto_pk_public_checksig(env,buf,sig,siglen);
@@ -936,12 +873,11 @@ crypto_pk_private_hybrid_decrypt(crypto_pk_env_t *env,
                                  size_t fromlen,
                                  int padding, int warnOnFailure)
 {
-  int overhead, outlen, r;
+  int outlen, r;
   size_t pkeylen;
   crypto_cipher_env_t *cipher = NULL;
   char buf[PK_BYTES+1];
 
-  overhead = crypto_get_rsa_padding_overhead(crypto_get_rsa_padding(padding));
   pkeylen = crypto_pk_keysize(env);
 
   if (fromlen <= pkeylen) {
@@ -1079,11 +1015,10 @@ crypto_pk_get_fingerprint(crypto_pk_env_t *pk, char *fp_out, int add_space)
   }
   base16_encode(hexdigest,sizeof(hexdigest),digest,DIGEST_LEN);
   if (add_space) {
-    if (tor_strpartition(fp_out, FINGERPRINT_LEN+1, hexdigest, " ", 4,
-                         NEVER_TERMINATE)<0)
+    if (tor_strpartition(fp_out, FINGERPRINT_LEN+1, hexdigest, " ", 4)<0)
       return -1;
   } else {
-    strcpy(fp_out, hexdigest);
+    strncpy(fp_out, hexdigest, HEX_DIGEST_LEN+1);
   }
   return 0;
 }
@@ -1632,7 +1567,7 @@ crypto_seed_rng(void)
   if (!provider_set) {
     if (!CryptAcquireContext(&provider, NULL, NULL, PROV_RSA_FULL,
                              CRYPT_VERIFYCONTEXT)) {
-      if (GetLastError() != NTE_BAD_KEYSET) {
+      if ((unsigned long)GetLastError() != (unsigned long)NTE_BAD_KEYSET) {
         log_warn(LD_CRYPTO, "Can't get CryptoAPI provider [1]");
         return rand_poll_status ? 0 : -1;
       }
@@ -1702,6 +1637,28 @@ crypto_rand_int(unsigned int max)
   }
 }
 
+/** Return a pseudorandom integer, chosen uniformly from the values
+ * between 0 and max-1. */
+uint64_t
+crypto_rand_uint64(uint64_t max)
+{
+  uint64_t val;
+  uint64_t cutoff;
+  tor_assert(max < UINT64_MAX);
+  tor_assert(max > 0); /* don't div by 0 */
+
+  /* We ignore any values that are >= 'cutoff,' to avoid biasing the
+   * distribution with clipping at the upper end of unsigned int's
+   * range.
+   */
+  cutoff = UINT64_MAX - (UINT64_MAX%max);
+  while (1) {
+    crypto_rand((char*)&val, sizeof(val));
+    if (val < cutoff)
+      return val % max;
+  }
+}
+
 /** Return a randomly chosen element of sl; or NULL if sl is empty.
  */
 void *
@@ -1771,6 +1728,9 @@ base64_decode(char *dest, size_t destlen, const char *src, size_t srclen)
   return ret;
 }
 
+/** Base-64 encode DIGEST_LINE bytes from <b>digest</b>, remove the trailing =
+ * and newline characters, and store the nul-terminated result in the first
+ * BASE64_DIGEST_LEN+1 bytes of <b>d64</b>.  */
 int
 digest_to_base64(char *d64, const char *digest)
 {
@@ -1781,6 +1741,9 @@ digest_to_base64(char *d64, const char *digest)
   return 0;
 }
 
+/** Given a base-64 encoded, nul-terminated digest in <b>d64</b> (without
+ * trailing newline or = characters), decode it and store the result in the
+ * first DIGEST_LEN bytes at <b>digest</b>. */
 int
 digest_from_base64(char *digest, const char *d64)
 {
@@ -1863,9 +1826,12 @@ secret_to_key(char *key_out, size_t key_out_len, const char *secret,
 }
 
 #ifdef TOR_IS_MULTITHREADED
+/** Helper: openssl uses this callback to manipulate mutexes. */
 static void
 _openssl_locking_cb(int mode, int n, const char *file, int line)
 {
+  (void)file;
+  (void)line;
   if (!_openssl_mutexes)
     /* This is not a really good  fix for the
      * "release-freed-lock-from-separate-thread-on-shutdown" problem, but
@@ -1877,6 +1843,8 @@ _openssl_locking_cb(int mode, int n, const char *file, int line)
     tor_mutex_release(_openssl_mutexes[n]);
 }
 
+/** Helper: Construct mutexes, and set callbacks to help OpenSSL handle being
+ * multithreaded. */
 static int
 setup_openssl_threading(void)
 {
