@@ -1,9 +1,9 @@
 /* Copyright (c) 2001 Matej Pfajfar.
  * Copyright (c) 2001-2004, Roger Dingledine.
- * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson. */
+ * Copyright (c) 2004-2007, Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: log.c 8479 2006-09-24 17:20:41Z nickm $ */
-const char log_c_id[] = "$Id: log.c 8479 2006-09-24 17:20:41Z nickm $";
+/* $Id: log.c 9612 2007-02-21 05:57:12Z nickm $ */
+const char log_c_id[] = "$Id: log.c 9612 2007-02-21 05:57:12Z nickm $";
 
 /**
  * \file log.c
@@ -13,6 +13,7 @@ const char log_c_id[] = "$Id: log.c 8479 2006-09-24 17:20:41Z nickm $";
 #include "orconfig.h"
 #include <stdarg.h>
 #include <assert.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #ifdef HAVE_SYS_TIME_H
@@ -60,12 +61,34 @@ sev_to_string(int severity)
   }
 }
 
+/** Helper: decide whether to include the function name in the log message. */
+static INLINE int
+should_log_function_name(uint32_t domain, int severity)
+{
+  switch (severity) {
+    case LOG_DEBUG:
+    case LOG_INFO:
+      /* All debugging messages occur in interesting places. */
+      return 1;
+    case LOG_NOTICE:
+    case LOG_WARN:
+    case LOG_ERR:
+      /* We care about places where bugs occur. */
+      return (domain == LD_BUG);
+    default:
+      assert(0); return 0;
+  }
+}
+
 /** Linked list of logfile_t. */
 static logfile_t *logfiles = NULL;
 #ifdef HAVE_SYSLOG_H
 static int syslog_count = 0;
 #endif
 
+/* What's the lowest log level anybody cares about? */
+int _log_global_min_severity = LOG_NOTICE;
+
 static void delete_log(logfile_t *victim);
 static void close_log(logfile_t *victim);
 
@@ -137,7 +160,7 @@ log_tor_version(logfile_t *lf, int reset)
  */
 static INLINE char *
 format_msg(char *buf, size_t buf_len,
-           int severity, const char *funcname,
+           uint32_t domain, int severity, const char *funcname,
            const char *format, va_list ap)
 {
   size_t n;
@@ -150,7 +173,7 @@ format_msg(char *buf, size_t buf_len,
   n = _log_prefix(buf, buf_len, severity);
   end_of_prefix = buf+n;
 
-  if (funcname) {
+  if (funcname && should_log_function_name(domain, severity)) {
     r = tor_snprintf(buf+n, buf_len-n, "%s(): ", funcname);
     if (r<0)
       n = strlen(buf);
@@ -206,11 +229,13 @@ logv(int severity, uint32_t domain, const char *funcname, const char *format,
 
     if (!formatted) {
       end_of_prefix =
-        format_msg(buf, sizeof(buf), severity, funcname, format, ap);
+        format_msg(buf, sizeof(buf), domain, severity, funcname, format, ap);
       formatted = 1;
     }
     if (lf->is_syslog) {
 #ifdef HAVE_SYSLOG_H
+      /* XXXX Some syslog implementations have scary limits on the length of
+       * what you can pass them.  Can/should we detect this? */
       syslog(severity, "%s", end_of_prefix);
 #endif
       lf = lf->next;
@@ -327,9 +352,12 @@ close_logs(void)
 }
 
 /** Remove and free the log entry <b>victim</b> from the linked-list
- * logfiles (it must be present in the list when this function is
- * called). After this function is called, the caller shouldn't refer
- * to <b>victim</b> anymore.
+ * logfiles (it is probably present, but it might not be due to thread
+ * racing issues). After this function is called, the caller shouldn't
+ * refer to <b>victim</b> anymore.
+ *
+ * Long-term, we need to do something about races in the log subsystem
+ * in general. See bug 222 for more details.
  */
 static void
 delete_log(logfile_t *victim)
@@ -380,6 +408,8 @@ add_stream_log(int loglevelMin, int loglevelMax,
   lf->file = stream;
   lf->next = logfiles;
   logfiles = lf;
+
+  _log_global_min_severity = get_min_log_level();
 }
 
 /** Add a log handler to receive messages during startup (before the real
@@ -390,6 +420,8 @@ add_temp_log(void)
 {
   add_stream_log(LOG_NOTICE, LOG_ERR, "<temp>", stdout);
   logfiles->is_temporary = 1;
+
+  _log_global_min_severity = get_min_log_level();
 }
 
 /**
@@ -408,9 +440,13 @@ add_callback_log(int loglevelMin, int loglevelMax, log_callback cb)
   lf->callback = cb;
   lf->next = logfiles;
   logfiles = lf;
+
+  _log_global_min_severity = get_min_log_level();
   return 0;
 }
 
+/** Adjust the configured severity of any logs whose callback function is
+ * <b>cb</b>. */
 void
 change_callback_log_severity(int loglevelMin, int loglevelMax,
                              log_callback cb)
@@ -422,6 +458,8 @@ change_callback_log_severity(int loglevelMin, int loglevelMax,
       lf->max_loglevel = loglevelMax;
     }
   }
+
+  _log_global_min_severity = get_min_log_level();
 }
 
 /** Close any log handlers added by add_temp_log or marked by mark_logs_temp */
@@ -441,6 +479,8 @@ close_temp_logs(void)
       p = &((*p)->next);
     }
   }
+
+  _log_global_min_severity = get_min_log_level();
 }
 
 /** Make all currently temporary logs (set to be closed by close_temp_logs)
@@ -479,6 +519,7 @@ add_file_log(int loglevelMin, int loglevelMax, const char *filename)
   if (log_tor_version(logfiles, 0) < 0) {
     delete_log(logfiles);
   }
+  _log_global_min_severity = get_min_log_level();
   return 0;
 }
 
@@ -501,6 +542,8 @@ add_syslog_log(int loglevelMin, int loglevelMax)
   lf->is_syslog = 1;
   lf->next = logfiles;
   logfiles = lf;
+
+  _log_global_min_severity = get_min_log_level();
   return 0;
 }
 #endif
@@ -554,7 +597,10 @@ switch_logs_debug(void)
 }
 
 #ifdef HAVE_EVENT_SET_LOG_CALLBACK
+/** A string which, if it appears in a libevent log, should be ignored. */
 static const char *suppress_msg = NULL;
+/** Callback function passed to event_set_log() so we can intercept
+ * log messages from libevent. */
 static void
 libevent_logging_callback(int severity, const char *msg)
 {
@@ -585,11 +631,13 @@ libevent_logging_callback(int severity, const char *msg)
       break;
   }
 }
+/** Set hook to intercept log messages from libevent. */
 void
 configure_libevent_logging(void)
 {
   event_set_log_callback(libevent_logging_callback);
 }
+/** Ignore any libevent log message that contains <b>msg</b>. */
 void
 suppress_libevent_log_msg(const char *msg)
 {
