@@ -1,9 +1,9 @@
 /* Copyright 2004 Roger Dingledine */
-/* Copyright 2004-2006 Roger Dingledine, Nick Mathewson */
+/* Copyright 2004-2007 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: torgzip.c 8479 2006-09-24 17:20:41Z nickm $ */
+/* $Id: torgzip.c 9634 2007-02-24 07:50:38Z arma $ */
 const char torgzip_c_id[] =
-  "$Id: torgzip.c 8479 2006-09-24 17:20:41Z nickm $";
+  "$Id: torgzip.c 9634 2007-02-24 07:50:38Z arma $";
 
 /**
  * \file torgzip.c
@@ -15,7 +15,7 @@ const char torgzip_c_id[] =
 #include <stdlib.h>
 #include <stdio.h>
 #include <assert.h>
-#ifdef MS_WINDOWS
+#ifdef _MSC_VER
 #include "..\..\contrib\zlib\zlib.h"
 #else
 #include <zlib.h>
@@ -29,6 +29,8 @@ const char torgzip_c_id[] =
 #include "log.h"
 #include "torgzip.h"
 
+/** Set to 1 if zlib is a version that supports gzip; set to 0 if it doesn't;
+ * set to -1 if we haven't checked yet. */
 static int gzip_is_supported = -1;
 
 /** Return true iff we support gzip-based compression.  Otherwise, we need to
@@ -49,6 +51,7 @@ is_gzip_supported(void)
   return gzip_is_supported;
 }
 
+/** Return the 'bits' value to tell zlib to use <b>method</b>.*/
 static INLINE int
 method_bits(compress_method_t method)
 {
@@ -76,7 +79,7 @@ tor_gzip_compress(char **out, size_t *out_len,
 
   if (method == GZIP_METHOD && !is_gzip_supported()) {
     /* Old zlib version don't support gzip in deflateInit2 */
-    log_warn(LD_GENERAL, "Gzip not supported with zlib %s", ZLIB_VERSION);
+    log_warn(LD_BUG, "Gzip not supported with zlib %s", ZLIB_VERSION);
     return -1;
   }
 
@@ -118,7 +121,12 @@ tor_gzip_compress(char **out, size_t *out_len,
         out_size *= 2;
         *out = tor_realloc(*out, out_size);
         stream->next_out = (unsigned char*)(*out + offset);
-        stream->avail_out = out_size - offset;
+        if (out_size - offset > UINT_MAX) {
+          log_warn(LD_BUG,  "Ran over unsigned int limit of zlib while "
+                   "uncompressing.");
+          goto err;
+        }
+        stream->avail_out = (unsigned int)(out_size - offset);
         break;
       default:
         log_warn(LD_GENERAL, "Gzip compression didn't finish: %s",
@@ -129,7 +137,7 @@ tor_gzip_compress(char **out, size_t *out_len,
  done:
   *out_len = stream->total_out;
   if (deflateEnd(stream)!=Z_OK) {
-    log_warn(LD_GENERAL, "Error freeing gzip structures");
+    log_warn(LD_BUG, "Error freeing gzip structures");
     goto err;
   }
   tor_free(stream);
@@ -152,6 +160,10 @@ tor_gzip_compress(char **out, size_t *out_len,
  * buffer, using the method described in <b>method</b>.  Store the uncompressed
  * string in *<b>out</b>, and its length in *<b>out_len</b>.  Return 0 on
  * success, -1 on failure.
+ *
+ * If <b>complete_only</b> is true, we consider a truncated input as a
+ * failure; otherwise we decompress as much as we can.  Warn about truncated
+ * or corrupt inputs at <b>protocol_warn_level</b>.
  */
 int
 tor_gzip_uncompress(char **out, size_t *out_len,
@@ -171,7 +183,7 @@ tor_gzip_uncompress(char **out, size_t *out_len,
 
   if (method == GZIP_METHOD && !is_gzip_supported()) {
     /* Old zlib version don't support gzip in inflateInit2 */
-    log_warn(LD_GENERAL, "Gzip not supported with zlib %s", ZLIB_VERSION);
+    log_warn(LD_BUG, "Gzip not supported with zlib %s", ZLIB_VERSION);
     return -1;
   }
 
@@ -206,11 +218,11 @@ tor_gzip_uncompress(char **out, size_t *out_len,
           goto done;
         /* There may be more compressed data here. */
         if ((r = inflateEnd(stream)) != Z_OK) {
-          log_warn(LD_GENERAL, "Error freeing gzip structures");
+          log_warn(LD_BUG, "Error freeing gzip structures");
           goto err;
         }
         if (inflateInit2(stream, method_bits(method)) != Z_OK) {
-          log_warn(LD_GENERAL, "Error from inflateInit2: %s",
+          log_warn(LD_GENERAL, "Error from second inflateInit2: %s",
                    stream->msg?stream->msg:"<no message>");
           goto err;
         }
@@ -231,7 +243,12 @@ tor_gzip_uncompress(char **out, size_t *out_len,
         out_size *= 2;
         *out = tor_realloc(*out, out_size);
         stream->next_out = (unsigned char*)(*out + offset);
-        stream->avail_out = out_size - offset;
+        if (out_size - offset > UINT_MAX) {
+          log_warn(LD_BUG,  "Ran over unsigned int limit of zlib while "
+                   "uncompressing.");
+          goto err;
+        }
+        stream->avail_out = (unsigned int)(out_size - offset);
         break;
       default:
         log_warn(LD_GENERAL, "Gzip decompression returned an error: %s",
@@ -244,7 +261,7 @@ tor_gzip_uncompress(char **out, size_t *out_len,
   r = inflateEnd(stream);
   tor_free(stream);
   if (r != Z_OK) {
-    log_warn(LD_GENERAL, "Error freeing gzip structures");
+    log_warn(LD_BUG, "Error freeing gzip structures");
     goto err;
   }
 
@@ -267,9 +284,9 @@ tor_gzip_uncompress(char **out, size_t *out_len,
 
 /** Try to tell whether the <b>in_len</b>-byte string in <b>in</b> is likely
  * to be compressed or not.  If it is, return the likeliest compression method.
- * Otherwise, return 0.
+ * Otherwise, return UNKNOWN_METHOD.
  */
-int
+compress_method_t
 detect_compression_method(const char *in, size_t in_len)
 {
   if (in_len > 2 && !memcmp(in, "\x1f\x8b", 2)) {
@@ -278,7 +295,114 @@ detect_compression_method(const char *in, size_t in_len)
              (ntohs(get_uint16(in)) % 31) == 0) {
     return ZLIB_METHOD;
   } else {
-    return 0;
+    return UNKNOWN_METHOD;
   }
 }
 
+/** Internal state for an incremental zlib compression/decompression.  The
+ * body of this struct is not exposed. */
+struct tor_zlib_state_t {
+  struct z_stream_s stream;
+  int compress;
+};
+
+/** Construct and return a tor_zlib_state_t object using <b>method</b>.  If
+ * <b>compress</b>, it's for compression; otherwise it's for
+ * decompression. */
+tor_zlib_state_t *
+tor_zlib_new(int compress, compress_method_t method)
+{
+  tor_zlib_state_t *out;
+
+  if (method == GZIP_METHOD && !is_gzip_supported()) {
+    /* Old zlib version don't support gzip in inflateInit2 */
+    log_warn(LD_BUG, "Gzip not supported with zlib %s", ZLIB_VERSION);
+    return NULL;
+ }
+
+ out = tor_malloc_zero(sizeof(tor_zlib_state_t));
+ out->stream.zalloc = Z_NULL;
+ out->stream.zfree = Z_NULL;
+ out->stream.opaque = NULL;
+ out->compress = compress;
+ if (compress) {
+   if (deflateInit2(&out->stream, Z_BEST_COMPRESSION, Z_DEFLATED,
+                    method_bits(method), 8, Z_DEFAULT_STRATEGY) != Z_OK)
+     goto err;
+ } else {
+   if (inflateInit2(&out->stream, method_bits(method)) != Z_OK)
+     goto err;
+ }
+ return out;
+
+ err:
+ tor_free(out);
+ return NULL;
+}
+
+/** Compress/decommpress some bytes using <b>state</b>.  Read up to
+ * *<b>in_len</b> bytes from *<b>in</b>, and write up to *<b>out_len</b> bytes
+ * to *<b>out</b>, adjusting the values as we go.  If <b>finish</b> is true,
+ * we've reached the end of the input.
+ *
+ * Return TOR_ZLIB_DONE if we've finished the entire compression/decompression.
+ * Return TOR_ZLIB_OK if we're processed everything from the input.
+ * Return TOR_ZLIB_BUF_FULL if we're out of space on <b>out</b>.
+ * Return TOR_ZLIB_ERR if the stream is corrupt.
+ */
+tor_zlib_output_t
+tor_zlib_process(tor_zlib_state_t *state,
+                 char **out, size_t *out_len,
+                 const char **in, size_t *in_len,
+                 int finish)
+{
+  int err;
+  state->stream.next_in = (unsigned char*) *in;
+  state->stream.avail_in = *in_len;
+  state->stream.next_out = (unsigned char*) *out;
+  state->stream.avail_out = *out_len;
+
+  if (state->compress) {
+    err = deflate(&state->stream, finish ? Z_FINISH : Z_SYNC_FLUSH);
+  } else {
+    err = inflate(&state->stream, finish ? Z_FINISH : Z_SYNC_FLUSH);
+  }
+
+  *out = (char*) state->stream.next_out;
+  *out_len = state->stream.avail_out;
+  *in = (const char *) state->stream.next_in;
+  *in_len = state->stream.avail_in;
+
+  switch (err)
+    {
+    case Z_STREAM_END:
+      return TOR_ZLIB_DONE;
+    case Z_BUF_ERROR:
+      if (state->stream.avail_in == 0)
+        return TOR_ZLIB_OK;
+      return TOR_ZLIB_BUF_FULL;
+    case Z_OK:
+      if (state->stream.avail_out == 0 || finish)
+        return TOR_ZLIB_BUF_FULL;
+      return TOR_ZLIB_OK;
+    default:
+      log_warn(LD_GENERAL, "Gzip returned an error: %s",
+               state->stream.msg ? state->stream.msg : "<no message>");
+      return TOR_ZLIB_ERR;
+    }
+}
+
+/** Deallocate <b>state</b>. */
+void
+tor_zlib_free(tor_zlib_state_t *state)
+{
+  tor_assert(state);
+
+  if (state->compress)
+    deflateEnd(&state->stream);
+  else
+    inflateEnd(&state->stream);
+
+  tor_free(state);
+}
+
