@@ -1,8 +1,8 @@
 /* Copyright 2003 Roger Dingledine
  * Copyright 2004-2007 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: util.c 9688 2007-02-28 20:24:27Z nickm $ */
-const char util_c_id[] = "$Id: util.c 9688 2007-02-28 20:24:27Z nickm $";
+/* $Id: util.c 10301 2007-05-24 15:48:53Z nickm $ */
+const char util_c_id[] = "$Id: util.c 10301 2007-05-24 15:48:53Z nickm $";
 
 /**
  * \file util.c
@@ -418,6 +418,36 @@ eat_whitespace(const char *s)
   }
 }
 
+/** Return a pointer to the first char of s before <b>eos</b> that is not
+ * whitespace and not a comment, or to the terminating NUL or eos if no such
+ * character exists.
+ */
+const char *
+eat_whitespace_eos(const char *s, const char *eos)
+{
+  tor_assert(s);
+  tor_assert(eos && s <= eos);
+
+  while (s < eos) {
+    switch (*s) {
+    case '\0':
+    default:
+      return s;
+    case ' ':
+    case '\t':
+    case '\n':
+    case '\r':
+      ++s;
+      break;
+    case '#':
+      ++s;
+      while (s < eos && *s && *s != '\n')
+        ++s;
+    }
+  }
+  return s;
+}
+
 /** Return a pointer to the first char of s that is not a space or a tab,
  * or to the terminating NUL if no such character exists. */
 const char *
