@@ -2,9 +2,9 @@
  * Copyright (c) 2001-2004, Roger Dingledine.
  * Copyright (c) 2004-2007, Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: circuitlist.c 9627 2007-02-24 01:12:53Z arma $ */
+/* $Id: circuitlist.c 10814 2007-07-12 16:50:41Z nickm $ */
 const char circuitlist_c_id[] =
-  "$Id: circuitlist.c 9627 2007-02-24 01:12:53Z arma $";
+  "$Id: circuitlist.c 10814 2007-07-12 16:50:41Z nickm $";
 
 /**
  * \file circuitlist.c
@@ -884,9 +884,9 @@ _circuit_mark_for_close(circuit_t *circ, int reason, int line,
                file, line, circ->purpose);
     }
     reason = END_CIRC_REASON_NONE;
-  } else if (CIRCUIT_IS_ORIGIN(circ) && reason < _END_CIRC_REASON_MIN) {
-    /* We don't send reasons when closing circuits at the origin, but we want
-     * to track them anyway so we can give them to the controller. */
+  }
+  if (CIRCUIT_IS_ORIGIN(circ)) {
+    /* We don't send reasons when closing circuits at the origin. */
     reason = END_CIRC_REASON_NONE;
   }
 
