@@ -2,9 +2,9 @@
  * Copyright (c) 2001-2004, Roger Dingledine.
  * Copyright (c) 2004-2007, Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: connection.c 10801 2007-07-12 16:17:05Z nickm $ */
+/* $Id: connection.c 11135 2007-08-16 16:41:48Z nickm $ */
 const char connection_c_id[] =
-  "$Id: connection.c 10801 2007-07-12 16:17:05Z nickm $";
+  "$Id: connection.c 11135 2007-08-16 16:41:48Z nickm $";
 
 /**
  * \file connection.c
@@ -132,10 +132,7 @@ conn_state_to_string(int type, int state)
       break;
     case CONN_TYPE_CONTROL:
       switch (state) {
-        case CONTROL_CONN_STATE_OPEN_V0: return "open (protocol v0)";
         case CONTROL_CONN_STATE_OPEN_V1: return "open (protocol v1)";
-        case CONTROL_CONN_STATE_NEEDAUTH_V0:
-          return "waiting for authentication (protocol unknown)";
         case CONTROL_CONN_STATE_NEEDAUTH_V1:
           return "waiting for authentication (protocol v1)";
       }
@@ -860,7 +857,7 @@ connection_init_accepted_conn(connection_t *conn, uint8_t listener_type)
       conn->state = DIR_CONN_STATE_SERVER_COMMAND_WAIT;
       break;
     case CONN_TYPE_CONTROL:
-      conn->state = CONTROL_CONN_STATE_NEEDAUTH_V0;
+      conn->state = CONTROL_CONN_STATE_NEEDAUTH_V1;
       break;
   }
   return 0;
@@ -2121,8 +2118,7 @@ connection_state_is_open(connection_t *conn)
       (conn->type == CONN_TYPE_AP && conn->state == AP_CONN_STATE_OPEN) ||
       (conn->type == CONN_TYPE_EXIT && conn->state == EXIT_CONN_STATE_OPEN) ||
       (conn->type == CONN_TYPE_CONTROL &&
-       (conn->state == CONTROL_CONN_STATE_OPEN_V0 ||
-        conn->state == CONTROL_CONN_STATE_OPEN_V1)))
+       conn->state == CONTROL_CONN_STATE_OPEN_V1))
     return 1;
 
   return 0;
