@@ -2,9 +2,9 @@
  * Copyright (c) 2001-2004, Roger Dingledine.
  * Copyright (c) 2004-2007, Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: connection_edge.c 10155 2007-05-10 08:57:57Z arma $ */
+/* $Id: connection_edge.c 10811 2007-07-12 16:17:31Z nickm $ */
 const char connection_edge_c_id[] =
-  "$Id: connection_edge.c 10155 2007-05-10 08:57:57Z arma $";
+  "$Id: connection_edge.c 10811 2007-07-12 16:17:31Z nickm $";
 
 /**
  * \file connection_edge.c
@@ -1685,10 +1685,14 @@ connection_ap_process_natd(edge_connection_t *conn)
   }
 
   daddr = tbuf = &tmp_buf[0] + 6; /* after end of "[DEST " */
-  while (*tbuf != '\0' && *tbuf != ' ')
-    tbuf++;
-  *tbuf = '\0';
-  tbuf++;
+  if (!(tbuf = strchr(tbuf, ' '))) {
+    log_warn(LD_APP,"Natd handshake was ill-formed; closing. The client "
+             "said: %s",
+             escaped(tmp_buf));
+    connection_mark_unattached_ap(conn, END_STREAM_REASON_INVALID_NATD_DEST);
+    return -1;
+  }
+  *tbuf++ = '\0';
 
   /* pretend that a socks handshake completed so we don't try to
    * send a socks reply down a natd conn */
@@ -2204,8 +2208,6 @@ connection_exit_begin_conn(cell_t *cell, circuit_t *circ)
       relay_send_command_from_edge(rh.stream_id, circ, RELAY_COMMAND_END,
                                    end_payload, 1, NULL);
       connection_free(TO_CONN(n_stream));
-      /* knock the whole thing down, somebody screwed up */
-      circuit_mark_for_close(circ, END_CIRC_REASON_CONNECTFAILED);
       tor_free(address);
       return 0;
     }
