@@ -1,9 +1,9 @@
 /* Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2007 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: directory.c 10301 2007-05-24 15:48:53Z nickm $ */
+/* $Id: directory.c 10808 2007-07-12 16:17:25Z nickm $ */
 const char directory_c_id[] =
-  "$Id: directory.c 10301 2007-05-24 15:48:53Z nickm $";
+  "$Id: directory.c 10808 2007-07-12 16:17:25Z nickm $";
 
 #include "or.h"
 
@@ -1361,7 +1361,7 @@ write_http_status_line(dir_connection_t *conn, int status,
 {
   char buf[256];
   if (tor_snprintf(buf, sizeof(buf), "HTTP/1.0 %d %s\r\n\r\n",
-      status, reason_phrase) < 0) {
+                   status, reason_phrase ? reason_phrase : "OK") < 0) {
     log_warn(LD_BUG,"Bug: status line too long.");
     return;
   }
@@ -1853,7 +1853,7 @@ directory_handle_command_post(dir_connection_t *conn, const char *headers,
   log_debug(LD_DIRSERV,"rewritten url as '%s'.", url);
 
   if (!strcmp(url,"/tor/")) { /* server descriptor post */
-    const char *msg;
+    const char *msg = NULL;
     int r = dirserv_add_descriptor(body, &msg);
     tor_assert(msg);
     if (r > 0)
