@@ -1,9 +1,9 @@
 /* Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2007 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: dirserv.c 9842 2007-03-16 06:58:19Z arma $ */
+/* $Id: dirserv.c 10161 2007-05-10 19:29:46Z nickm $ */
 const char dirserv_c_id[] =
-  "$Id: dirserv.c 9842 2007-03-16 06:58:19Z arma $";
+  "$Id: dirserv.c 10161 2007-05-10 19:29:46Z nickm $";
 
 #include "or.h"
 
@@ -610,6 +610,7 @@ directory_remove_invalid(void)
                ent->nickname, msg?msg:"");
       routerlist_remove(rl, ent, i--, 0);
       changed = 1;
+      continue;
     }
     if (bool_neq((r & FP_NAMED), ent->is_named)) {
       log_info(LD_DIRSERV,
@@ -1678,6 +1679,8 @@ generate_v2_networkstatus(void)
       outp += strlen(outp);
       if (ri->platform && !strcmpstart(ri->platform, "Tor ")) {
         const char *eos = find_whitespace(ri->platform+4);
+        if (eos && !strcmpstart(eos, " (r"))
+          eos = find_whitespace(eos+1);
         if (eos) {
           char *platform = tor_strndup(ri->platform, eos-(ri->platform));
           if (tor_snprintf(outp, endp-outp,
