@@ -1,9 +1,9 @@
 /* Copyright 2003-2004 Roger Dingledine.
  * Copyright 2004-2007 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: dns.c 9625 2007-02-23 21:56:10Z nickm $ */
+/* $Id: dns.c 10243 2007-05-22 01:39:31Z nickm $ */
 const char dns_c_id[] =
-  "$Id: dns.c 9625 2007-02-23 21:56:10Z nickm $";
+  "$Id: dns.c 10243 2007-05-22 01:39:31Z nickm $";
 
 /**
  * \file dns.c
@@ -1519,6 +1519,7 @@ configure_nameservers(int force)
   or_options_t *options;
   const char *conf_fname;
   struct stat st;
+  int r;
   options = get_options();
   conf_fname = options->ServerDNSResolvConfFile;
 #ifndef MS_WINDOWS
@@ -1543,9 +1544,9 @@ configure_nameservers(int force)
       evdns_clear_nameservers_and_suspend();
     }
     log_info(LD_EXIT, "Parsing resolver configuration in '%s'", conf_fname);
-    if (evdns_resolv_conf_parse(DNS_OPTIONS_ALL, conf_fname)) {
-      log_warn(LD_EXIT, "Unable to parse '%s', or no nameservers in '%s'",
-               conf_fname, conf_fname);
+    if ((r = evdns_resolv_conf_parse(DNS_OPTIONS_ALL, conf_fname))) {
+      log_warn(LD_EXIT, "Unable to parse '%s', or no nameservers in '%s' (%d)",
+               conf_fname, conf_fname, r);
       return -1;
     }
     if (evdns_count_nameservers() == 0) {
